#!/usr/bin/env python
# -*- mode: landisk-python; coding: utf-8; -*-
# vim:ts=4 sw=4 sts=4 ai si et sta

"""PAL command for Dixim DMS package."""


import os
import shutil
import glob

from iobase.exception import (InvalidArgError, IoBaseException, NotFoundError)
from iobase.xmldata import (XmlConfNode,)
from iobase.cmdexec import (CmdExec, NULL)
from pal_info import (ReqTypes, Results)
from albg_info import (ALBG_DIXIMDMS_PATH,)
import pal_config as config

import libpal.palcmd as palcmd
from libpal.util import RemountRoot

from pkgaddon.pal_diximdms_config import (EXEC_LOCK_NAME,
                                INIT_STATUS_FILE_PATH,
                                STATUS_NO_EXEC, STATUS_PROCESSING,
                                STATUS_ABORT, DMS_LANG_CONF_PATH,
                                DMS_SHARE_CONF_PATH, DMSCTRL_CMD,
                                DMS_DTCP_LANG_CONF_PATH,
                                DMS_DTCP_SHARE_CONF_PATH,
                                DMS_DTCP_UPLOAD_PATH, DMS_RC_SCRIPT,
                                DMSCTRL_DTCP_CMD, CHECK_LICENSE_CMD,
                                REMOVE_XSRS_CMD,
                                DMS_DTCP_LICENSE_PATH, DTCP_MTD_BLOCK_DEV,
                                DMS_DATABASE_PATH, DMS_DTCP_DATABASE_SRC_PATH,
                                DMS_DTCP_PREPARE_DIR_LIST,
                                DMS_DTCP_EXPORTS_PATH,)


DIXIMDMS_NAME = "diximdms"
LANG_NAME = "lang"
SHARE_PATH_PATH = "sharelist/share/path"

STATUS_PATH = "status"
TIMESTAMP_PATH = "lasttime/timestamp"

STATUSINFO_XML = """<statusinfo>
    <status/>
    <lasttime>
        <timestamp/>
    </lasttime>
</statusinfo>
"""

LICENSE_KEY_PATH = "dtcp/license/key"

NEEDREBOOT_ATTR = "needreboot"
LICENSE_NAME = "license"
KEY_NAME = "key"
DTCP_XML = "<dtcp><license/></dtcp>"

DMSCTRL_ARGS = [DMSCTRL_CMD, "init-cds"]
CHECK_LICENSE_ARGS = [CHECK_LICENSE_CMD, "register"]


class RequestError(IoBaseException):
    """Request error class."""

    pass


class PalDiximDms(palcmd.PalCmd):
    """Pal DiximDms command implementation class."""

    #
    # Internal methods
    #

    def _check_body_node(self, node, type=None):
        """Check if entry's body node is valid."""

        if node.get_node_name() != DIXIMDMS_NAME:
            raise InvalidArgError(node.get_node_name())

    def _is_albg_running(self):
        """Return if albg_migration is running."""

        try:
            lock = FLock(config.LOCK_DIR, EXEC_LOCK_NAME, exclusive=True,
                            timeout=0)
            lock.lock()
            lock.unlock()
            # a lock file is not locked. albg_eset is not running.
            return False
        except:
            # a lock file is locked. albg_eset is running.
            return True

    def _get_status(self, status_path):
        """Return the current status."""

        if not os.path.exists(status_path):
            return STATUS_NO_EXEC

        status = STATUS_PROCESSING
        if os.path.exists(status_path):
            with open(status_path) as status_file:
                status = status_file.read().strip()

        if status == STATUS_PROCESSING and not self._is_albg_running():
            return STATUS_ABORT

        return status

    def _try_udpate_lang(self, diximdms):
        """Try updating language setting."""

        try:
            lang = diximdms.get_node(LANG_NAME).get_text()
            if lang:
                for lang_conf_path in \
                        (DMS_LANG_CONF_PATH, DMS_DTCP_LANG_CONF_PATH):

                    with open(lang_conf_path, "w") as conf_file:
                        conf_file.write(lang)
                CmdExec(DMSCTRL_ARGS, stdout=NULL, stderr=NULL).execute()
        except NotFoundError:
            # "lang" is not specified.
            pass

    def _try_udpate_share(self, diximdms):
        """Try updating language setting."""

        # For DMS non DTCP-IP version
        with open(DMS_SHARE_CONF_PATH, "w") as dms_conf_file:
            for path in diximdms.get_node_list(SHARE_PATH_PATH):
                dms_conf_file.write(path.get_text() + os.linesep)

        # For DMS DTCP-IP version
        with open(DMS_DTCP_SHARE_CONF_PATH, "w") as dms_dtcp_conf_file:
            dms_dtcp_conf_file.write(DMS_DTCP_UPLOAD_PATH + os.linesep)

            for path in diximdms.get_node_list(SHARE_PATH_PATH):
                exports_path = os.path.join(DMS_DTCP_EXPORTS_PATH,
                                        os.path.basename(path.get_text()))
                dms_dtcp_conf_file.write(exports_path + os.linesep)

    def _save_license_key(self, key):
        """Save license key data."""

        if not os.path.isdir(os.path.dirname(DMS_DTCP_LICENSE_PATH)):
            os.makedirs(os.path.dirname(DMS_DTCP_LICENSE_PATH))

        with open(DMS_DTCP_LICENSE_PATH, "w") as license:
            license.write(key)

    def _load_license_key(self):
        """Load license key data and return it."""

        try:
            with open(DMS_DTCP_LICENSE_PATH) as license:
                return license.read().strip()
        except:
            return ""

    def _update_dms_rc_script(self):
        """Change the destination of symblic link file for DMS RC script."""

        with RemountRoot():
            os.remove(DMS_RC_SCRIPT)
            os.symlink(DMSCTRL_DTCP_CMD, DMS_RC_SCRIPT)

    def _prepare_for_dms_dtcp(self):
        """Prepare for diximdms service supporting DTCP-IP."""

        try:
            shutil.rmtree(DMS_DATABASE_PATH)
        except:
            # Ignore error
            pass

        if not os.path.exists(DMS_DATABASE_PATH):
            os.makedirs(DMS_DATABASE_PATH)

        for src_path in glob.glob(
                            os.path.join(DMS_DTCP_DATABASE_SRC_PATH, "*")):
            dest_path = os.path.join(DMS_DATABASE_PATH,
                                        os.path.basename(src_path))
            if os.path.exists(dest_path):
                if os.stat(src_path).st_mtime <= os.stat(dest_path).st_mtime:
                    continue

            shutil.copy2(src_path, dest_path)

        for prepare_dir in DMS_DTCP_PREPARE_DIR_LIST:
            if not os.path.exists(prepare_dir):
                os.makedirs(prepare_dir)

    #
    # Request handlers
    #

    def _query_handler(self, entry):
        """Handle query request."""

        status_info = XmlConfNode(STATUSINFO_XML)
        status = self._get_status(INIT_STATUS_FILE_PATH)
        status_info.get_node(STATUS_PATH).set_text(status)
        if os.path.exists(INIT_STATUS_FILE_PATH):
            last_time = int(os.stat(INIT_STATUS_FILE_PATH).st_mtime)
            status_info.get_node(TIMESTAMP_PATH).set_text(str(last_time))

        entry.get().append_child(status_info)

        dtcp = XmlConfNode(DTCP_XML)
        if os.path.exists(DTCP_MTD_BLOCK_DEV):
            dtcp.set_bool_attr(NEEDREBOOT_ATTR, False)
        else:
            dtcp.set_bool_attr(NEEDREBOOT_ATTR, True)

        license_key = self._load_license_key()
        if license_key:
            key = XmlConfNode(KEY_NAME)
            key.set_text(license_key)
            dtcp.get_node(LICENSE_NAME).append_child(key)

        entry.get().append_child(dtcp)

        entry.set_result(Results.OK)
        return entry

    def _apply_handler(self, entry):
        """Handle apply request."""

        self._check_body_node(entry.get())

        try:
            diximdms = entry.get()
            self._try_udpate_lang(diximdms)
            self._try_udpate_share(diximdms)
        except:
            self.exception("apply fail.")
            entry.set_result(Results.APPLY_ERROR)
            return entry

        entry.set_result(Results.OK)
        return entry

    def _initialize_handler(self, entry):
        """Handle initialize request."""

        try:
            response_entry = self.send_entry(entry, ALBG_DIXIMDMS_PATH)
            if response_entry.get_result() != Results.OK:
                raise RequestError(response_entry.get_result())
            entry.set_result(Results.OK)
        except:
            self.exception("Initialize fail.")
            entry.set_result(Results.INITIALIZE_ERROR)
        return entry

    def _register_handler(self, entry):
        """Handle register request."""

        self._check_body_node(entry.get())
        try:
            key = entry.get().get_node(LICENSE_KEY_PATH).get_text().strip()
            if key:
                CmdExec(CHECK_LICENSE_ARGS + [key],
                        stdout=NULL, stderr=NULL).execute()

                self._save_license_key(key)
                self._update_dms_rc_script()
                self._prepare_for_dms_dtcp()
                entry.set_result(Results.OK)
            else:
                self.error("key is not specified!!")
                entry.set_result(Results.INVALID_DATA)
        except NotFoundError:
            self.exception("Key not found.")
            entry.set_result(Results.INVALID_DATA)
        except:
            self.exception("execution fail.")
            entry.set_result(Results.EXECUTE_ERROR)
        return entry

    def _delete_handler(self, entry):
        """Handle delete request."""

        self._check_body_node(entry.get())
        try:
                CmdExec(REMOVE_XSRS_CMD, stdout=NULL, stderr=NULL).execute()
                entry.set_result(Results.OK)
        except:
            self.exception("execution fail.")
            entry.set_result(Results.DELETE_ERROR)
        return entry

    #
    # Constructor
    #

    def __init__(self, option=None):
        """Constructor."""

        palcmd.PalCmd.__init__(self, option=option)

        self.set_handler(ReqTypes.QUERY, self._query_handler)
        self.set_handler(ReqTypes.APPLY, self._apply_handler)
        self.set_handler(ReqTypes.INITIALIZE, self._initialize_handler)
        self.set_handler(ReqTypes.REGISTER, self._register_handler)
        self.set_handler(ReqTypes.DELETE, self._delete_handler)


#
# Main
#

if __name__ == "__main__":
    PalDiximDms(option=palcmd.PalCmdOption()).handle_request()
