/**
 * @module dixim
 * @submodule contents
 */
dixim.namespace('dixim.contents');

/**
 * @module contents
 * @submodule contents-controller
 */

(function() {
var ContentsController;
var prototype;
var superConstructor;
var ContentsView = dixim.contents.ContentsView;
var EventTarget = dixim.event.EventTarget;
var _ = dixim.ctt.ResourceBundle.get;
var PinManager = dixim.ctt.util.PinManager;
var ApiClientTransfer = dixim.common.ApiClientTransfer;
var OperationMenuDialogController = dixim.contents.OperationMenuDialogController;
var ErrorModalDialogView = dixim.common.ErrorModalDialogView;
var SettingMenuDialogController = dixim.contents.SettingMenuDialogController;
var SearchDialogController = dixim.contents.SearchDialogController;
var Enum = dixim.ctt.util.Enum;

/**
 * @class ContentsController
 * @constructor
 * @param {Object} configs
 */
ContentsController = function(configs) {
    EventTarget.call(this);
    this._initialize(configs);
};

dixim.inherit(ContentsController, EventTarget);
dixim.contents.ContentsController = ContentsController;
prototype = ContentsController.prototype;
superConstructor = ContentsController.superConstructor;

var SELECT = 'select';
var SELECT_CONTENT = 'selectContent';
var OPERATE = 'operate';
var REFRESH = 'refresh';
var SETTING = 'setting';
var SEARCH = 'search'
var PREV = 'prev';
var UPDATE = 'update';
var CHANGE_CHECKBOX = 'changeCheckbox';

var OperationMenu = Enum.fetch('OPERATION_MENU');
var ROOT_OBJECTID = 'FS-Folder';

var BROWSING_STATUS = {
    STOPPED: 0,
    STOPPING: 1,
    BROWSING: 2,
};

/**
 * @method _initialize
 * @protected
 * @param {Object} configs
 */
prototype._initialize = function(configs) {
    configs = configs || {};

    configs.id = configs.id || '';
    this._id = [configs.id, 'contents'].join('');

    this._model = new ApiClientTransfer({isCancelAll: true});
    this._model.addEventListener('browse', this._successBrowse, this);
    this._model.addEventListener('browseError', this._errorBrowse, this);
    this._model.addEventListener('search', this._successSearch, this);
    this._model.addEventListener('searchError', this._errorSearch, this);

    this._view = new ContentsView(configs);
    this._view.addEventListener(SELECT, this._clickSelect, this);
    this._view.addEventListener(SELECT_CONTENT, this._clickSelectContent, this);
    this._view.addEventListener(OPERATE, this._clickOperate, this);
    this._view.addEventListener(REFRESH, this._clickRefresh, this);
    this._view.addEventListener(PREV, this._clickPrev, this);
    this._view.addEventListener(SETTING, this._clickSetting, this);
    this._view.addEventListener(SEARCH, this._clickSearch, this);
    this._view.addEventListener(CHANGE_CHECKBOX, this._checkCheckbox, this);

    this._parentContext = configs.parentContext;
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._browseRequestData;
    this._browsingStack = [];
    this._browsingStackObj = [];
    this._operationStatus = OperationMenu.NO_SELECTED;
    this._rootObjectId = null;
    this._searchingTitle;
};

/**
 * @method start
 */
prototype.start = function() {
    var id = '0';

    this._view.show();
    this._browse(id);
};

/**
 * @method stop
 */
prototype.stop = function() {
    this._view.hide();
};

prototype._browse = function(id) {
    this._view.startLoading();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._browseRequest(id);
};

prototype._browseRequest = function(id, startingIndex) {
    var query = {};
    var pinCode = PinManager.get();

    query.id = id;
    query.starting_index = startingIndex || 0;
    query.requested_count = 20;
    if (pinCode !== null) query.pin_code = pinCode;

    this._browseRequestData = query;

    this._model.browse(query);
};

prototype._viewContents = function(results) {
    var resultsLen = results.length;

    for (var i = 0; i < resultsLen; i += 1) {
        this._view.addContent(results[i], (this._rootObjectId === null));
    }
    this._checkCheckbox();
};

prototype._successBrowse = function(e) {
    var totalReturned = e.args.numberReturned;
    var totalMatches = e.args.totalMatches;
    var startingIndex = e.args.startingIndex;
    var results = e.args.results;
    var isStopping = (this._browsingStatus === BROWSING_STATUS.STOPPING);
    var isEndBrowsing = ((totalReturned + startingIndex) === totalMatches);
    var isMatchId = true;

    if (results.length !== 0) isMatchId = (this._browseRequestData.id === results[0].parentId);

    if (totalMatches === 0) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.noContent();
        this._view.stopLoading();
        return;
    }
    if (isStopping || (!isMatchId)) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        return;
    }

    this._viewContents(results);

    if (isEndBrowsing) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.stopLoading();
        return;
    }

    totalReturned += this._browseRequestData.starting_index;
    this._browseRequest(this._browseRequestData.id, totalReturned);
};

prototype._errorBrowse = function() {
    this._errorDialog(_('contents_fetch_error'));
    this._browsingStatus = BROWSING_STATUS.STOPPED;
    this._clickPrev();
};

prototype._browseAfterStop = function(id) {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.STOPPING;
    }
   
    if (this._browsingStatus !== BROWSING_STATUS.STOPPED) {
        setTimeout(dixim.proxy(function() {
            this._browseAfterStop(id)
        }, this), 100);
    } else {
        this._view.cleanContent();
        this._browse(id);
    }
};

prototype._search = function(title) {
    this._view.startLoading();
    this._browsingStatus = BROWSING_STATUS.BROWSING;
    this._searchRequest(title, 0);
};

prototype._searchRequest = function(title, startingIndex) {
    var query = {};
    var pinCode = PinManager.get();

    query.id = '0';
    query.key = 'title';
    query.value = [title];
    query.starting_index = startingIndex || 0;
    query.requested_count = 20;
    query.filter = 'item';
    if (pinCode !== null) query.pin_code = pinCode;

    this._browseRequestData = query;

    this._model.search(query);
};

prototype._successSearch = function(e) {
    var totalReturned = e.args.numberReturned;
    var totalMatches = e.args.totalMatches;
    var startingIndex = e.args.startingIndex;
    var results = e.args.results;
    var resultsLen = results.length;
    var isStopping = (this._browsingStatus === BROWSING_STATUS.STOPPING);
    var isEndBrowsing = ((totalReturned + startingIndex) === totalMatches);
    var isMatch = true;

    for (var i = 0; i < resultsLen; i += 1) {
        if (results[i].title.toLowerCase().indexOf(this._browseRequestData.value[0].toLowerCase()) === -1) {
            isMatch = false;
            break;
        }
    }

    if (totalMatches === 0) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.noContent();
        this._view.stopLoading();
        return;
    }
    if (isStopping || (!isMatch)) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.stopLoading();
        return;
    }

    this._viewContents(results);

    if (isEndBrowsing) {
        this._browsingStatus = BROWSING_STATUS.STOPPED;
        this._view.stopLoading();
        return;
    }

    totalReturned += this._browseRequestData.starting_index;
    this._searchRequest(this._browseRequestData.value[0], totalReturned);
};

prototype._errorSearch = function() {
    this._errorDialog(_('contents_fetch_error'));
    this._browsingStatus = BROWSING_STATUS.STOPPED;
};

prototype._searchAfterStop = function(title) {
    if (this._browsingStatus === BROWSING_STATUS.BROWSING) {
        this._browsingStatus = BROWSING_STATUS.STOPPING;
    }
   
    if (this._browsingStatus !== BROWSING_STATUS.STOPPED) {
        setTimeout(dixim.proxy(function() {
            this._searchAfterStop(title)
        }, this), 100);
    } else {
        this._view.cleanContent();
        this._search(title);
    }
};

prototype._clickSelectContent = function(e) {
    var data = e.args;

    if (data.parentId === ROOT_OBJECTID) this._rootObjectId = data.id;
    this._browsingStack.push(data.parentId);
    this._browsingStackObj.push(data);
    this._browseAfterStop(data.id);
    this._view.setContentName(data.title);
    this._checkCheckbox();
};

prototype._clickSelect = function() {
    this._view.switchSelect();
    this._checkCheckbox();
};

prototype._clickOperate = function(e) {
    var query = {};
    var controller;
    var dataList = e.args.dataList;
    var len = dataList.length;

    this._operationStatus |= OperationMenu.NO_SELECTED;
    this._operationStatus &= ~(OperationMenu.MULTIPLE | OperationMenu.CONTAINER | OperationMenu.ITEM | OperationMenu.CATEGORY_CONTENTS);
    if (len !== 0) {
        this._operationStatus &= ~(OperationMenu.NO_SELECTED);
        if (len > 1) this._operationStatus |= OperationMenu.MULTIPLE;
        for (var i = 0; i < len; i += 1) {
            if (dataList[i].class === 'container') this._operationStatus |= OperationMenu.CONTAINER;
            if (dataList[i].class === 'item') this._operationStatus |= OperationMenu.ITEM;
        }
    }
    if (this._rootObjectId === null) this._operationStatus |= OperationMenu.CATEGORY_CONTENTS;

    query.dataList = dataList;
    query.parentId = this._browseRequestData.id;
    query.operationStatus = this._operationStatus;
    query.browsingStackObj = this._browsingStackObj;
    controller = new OperationMenuDialogController(query);
    controller.addEventListener(UPDATE, this._update , this);
    controller.start();
};

prototype._clickRefresh = function() {
    this._update();
    this._checkCheckbox();
};

prototype._clickPrev = function() {
    var data = this._browsingStack.pop();
    this._browsingStackObj.pop();

    if (typeof data === 'undefined') {
        this.stop();
        this._parentContext.start();
    } else if (typeof data === 'object') {
        if (typeof data.value === 'undefined') {
            this._operationStatus &= ~OperationMenu.SEARCH_PAGE;
            this._browseAfterStop(data.id);
        } else {
            this._operationStatus |= OperationMenu.SEARCH_PAGE;
            this._searchAfterStop(data.value[0]);
        }
        this._view.unsetContentName();
    } else {
        if (data === ROOT_OBJECTID) this._rootObjectId = null;
        this._browseAfterStop(data);
        this._view.unsetContentName();
    }
};

prototype._clickSetting = function() {
    var controller;

    controller = new SettingMenuDialogController();
    controller.start();
};

prototype._checkCheckbox = function() {
    this._operationStatus &= ~(OperationMenu.CATEGORY_CONTENTS);

    if (this._rootObjectId === null) this._operationStatus |= OperationMenu.CATEGORY_CONTENTS;

    this._view.checkActionBarStatus(this._operationStatus);
};

prototype._clickSearch = function() {
    var controller;

    controller = new SearchDialogController();
    controller.addEventListener(SEARCH, this._callbackSearch, this);
    controller.start();
};

prototype._callbackSearch = function(e) {
    this._operationStatus |= OperationMenu.SEARCH_PAGE;
    this._browsingStack.push(this._browseRequestData);
    this._browsingStackObj.push(null);
    this._searchingTitle = e.args.title;
    this._searchAfterStop(e.args.title);
    this._view.setContentName(_('contents_search_result', e.args.title));
};

prototype._errorDialog = function(text) {
    var query = {};
    var dialog;

    query.id = this._id;
    query.title = _('common_menu_title_contents');
    query.description = text;
    dialog = new ErrorModalDialogView(query);
    dialog.show();
};

prototype._update = function() {
    if (this._operationStatus & OperationMenu.SEARCH_PAGE) {
        this._searchAfterStop(this._searchingTitle);
    } else {
        this._browseAfterStop(this._browseRequestData.id);
    }
};

}());
